@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002 2003 2004 2005, Magnus Hjorth
+ * Copyright (C) 2002 2003 2004 2005 2006, Magnus Hjorth
  *
  * This file is part of mhWaveEdit.
  *
@@ -151,7 +151,7 @@ static gboolean chunk_convert_samplerate_proc(void *in, guint sample_size,
      gchar buf[4096];
      if (conv == NULL) 
 	  conv = rateconv_new(FALSE,conv_driver,informat,
-			      conv_outrate,conv_dither_mode);
+			      conv_outrate,conv_dither_mode,FALSE);
      if (conv == NULL) return TRUE;
      if (sample_size == 0) rateconv_write(conv,NULL,0);
      while (1) {
@@ -287,15 +287,16 @@ gboolean chunk_dump(Chunk *chunk, EFILE *file, gboolean bigendian,
 {
      GList *l;
      DataPart *dp;
+     off_t clipcount = 0;
      l = chunk->parts;
      while (l) {
 	  dp = (DataPart *)l->data;
 	  if (datasource_dump(dp->ds,dp->position,dp->length,file,
-			      dither_mode,bar))
+			      dither_mode,bar,&clipcount))
 	       return TRUE;
 	  l = l->next;
      }
-     return FALSE;
+     return clipwarn(clipcount,TRUE);
 }
 
 static gboolean has_fake_pcm(Chunk *c)
@@ -331,6 +332,7 @@ struct convert_back {
      Dataformat *tofmt;
      gchar buf[16384];
      int dither_mode;
+     off_t *clipcount;
 };
 
 static gboolean convert_back_write(WriteoutID id, gpointer data, guint length)
@@ -338,6 +340,7 @@ static gboolean convert_back_write(WriteoutID id, gpointer data, guint length)
      struct convert_back *cbp = (struct convert_back *)id;
      guint i;
      i = length / sizeof(sample_t);
+     *(cbp->clipcount) += unnormalized_count(data,i);
      convert_array(data,&dataformat_sample_t,cbp->buf,cbp->tofmt,i,
 		   cbp->dither_mode);
      return tempfile_write(cbp->tmp,cbp->buf,i*cbp->tofmt->samplesize);
@@ -349,6 +352,36 @@ Chunk *chunk_filter_tofmt(Chunk *chunk, chunk_filter_tofmt_proc proc,
 			  Dataformat *tofmt, int dither_mode,
 			  StatusBar *bar, gchar *title)
 {
+     /* This code has to keep track of five different sample formats:
+      *   1) The Chunk's native format (chunk->format)
+      *   2) The processing function's input format (informat)
+      *   3) The processing function's output format (outformat)
+      *   4) The output temporary file's format (outformat or tofmt)
+      *   5) The output Chunk's native format (tofmt)
+      *
+      * These modes are possible:
+      *
+      *   1) Native format all the way
+      *      if must_convert == FALSE and no FP data in chunk
+      *      convert = FALSE, convert_back = FALSE,
+      *      formats (1)=(2)=(3), (4)=(5)
+      *
+      *   2) FP processing with FP tempfiles
+      *      if must_convert == TRUE || FP data in chunk
+      *      convert = TRUE, convert_back = FALSE
+      *      formats (2)=(3)=floating point version of (1)
+      *      format (4)=floating point version of (5)
+      *
+      *   3) FP processing with native tempfiles
+      *      (this mode is only used if the "use FP tempfiles option" is 
+      *      disabled, could cause quality loss)
+      *      if (must_convert == TRUE || FP data in chunk) && 
+      *          !chunk_filter_use_floating_tempfiles,
+      *      convert = TRUE, convert_back = TRUE
+      *      formats (2)=(3)=floating point version of (1),  (4)=(5)
+      */
+
+
      guint proc_size,single_size,full_size;
      gchar buf[8192];
      off_t samplepos=0, samplesleft=chunk->length;
@@ -357,8 +390,9 @@ Chunk *chunk_filter_tofmt(Chunk *chunk, chunk_filter_tofmt_proc proc,
      Dataformat informat,outformat;
      ChunkHandle *ch;
      TempFile tmp;
-     Chunk *ds,*r;
+     Chunk *ds,*r;     
      struct convert_back *cbp = NULL;
+     off_t clipcount = 0;
      
      /* Force processing in floating-point if any of the Datasources has 
       * faked pcm data */
@@ -394,6 +428,7 @@ Chunk *chunk_filter_tofmt(Chunk *chunk, chunk_filter_tofmt_proc proc,
 	  cbp->tmp = tmp;
 	  cbp->tofmt = tofmt;
 	  cbp->dither_mode = dither_mode;
+	  cbp->clipcount = &clipcount;
      }
      status_bar_begin_progress(bar,samplesleft,title);
 
@@ -401,11 +436,13 @@ Chunk *chunk_filter_tofmt(Chunk *chunk, chunk_filter_tofmt_proc proc,
 	  if (convert)
 	       u = chunk_read_array_fp(ch,samplepos,
 				       MIN(sizeof(buf)/full_size,samplesleft),
-				       (sample_t *)buf,dither_mode)*full_size;
+				       (sample_t *)buf,dither_mode,
+				       &clipcount) *
+		    full_size;
 	  else
 	       u = chunk_read_array(ch,samplepos,
 				    MIN(sizeof(buf),samplesleft*full_size),buf,
-				    dither_mode);
+				    dither_mode,&clipcount);
 	  if (!u) {
 	       chunk_close(ch);
 	       tempfile_abort(tmp);
@@ -447,6 +484,12 @@ Chunk *chunk_filter_tofmt(Chunk *chunk, chunk_filter_tofmt_proc proc,
      chunk_close(ch);
      ds = tempfile_finished(tmp);
      status_bar_end_progress(bar);
+     if (cbp != NULL) g_free(cbp);
+
+     if (clipwarn(clipcount,TRUE)) {
+	  gtk_object_sink(GTK_OBJECT(ds));
+	  return NULL;
+     }
 
      /* Check if the datasources format is the same as the user expects. If 
       * not, convert. */
@@ -465,14 +508,25 @@ Chunk *chunk_filter_tofmt(Chunk *chunk, chunk_filter_tofmt_proc proc,
 
 gboolean chunk_parse(Chunk *chunk, chunk_parse_proc proc, 
 		     gboolean allchannels, gboolean convert, int dither_mode, 
-		     StatusBar *bar, gchar *title)
+		     StatusBar *bar, gchar *title, off_t samplepos, 
+		     gboolean reverse)
 {
 
      guint single_size,full_size,proc_size;
      gchar buf[4096],*c=NULL,*d;
-     off_t samplepos=0, samplesleft=chunk->length;
-     guint u, x;
+     off_t samplesleft;
+     off_t readpos;
+     off_t readsamples;
+     guint u;
+     gint x;
      ChunkHandle *ch;
+     off_t clipcount = 0;
+
+     if(reverse) {
+          samplesleft = samplepos;
+     } else {
+          samplesleft = chunk->length - samplepos;
+     }
 
      if (convert) single_size = sizeof(sample_t);
      else single_size = chunk->format.samplesize;
@@ -482,34 +536,76 @@ gboolean chunk_parse(Chunk *chunk, chunk_parse_proc proc,
      if (allchannels) proc_size = full_size;
      else proc_size = single_size;
 
+     readsamples = sizeof(buf) / full_size;
+     if(samplesleft < readsamples) {
+          readsamples = samplesleft;
+     }
 
      ch = chunk_open(chunk);
      if (ch == NULL) { g_free(c); return TRUE; }
      status_bar_begin_progress( bar, chunk->length, title);
 
      while (samplesleft > 0) {
-	  if (convert)	       
-	       u = chunk_read_array_fp(ch,samplepos,sizeof(buf)/full_size,
-				       (sample_t *)buf,dither_mode)*full_size;
-	  else
-	       u = chunk_read_array(ch,samplepos,sizeof(buf),buf,dither_mode);
+	  if(reverse) {
+	       readpos = samplepos - readsamples;
+	  } else {
+	       readpos = samplepos;
+	  }
+
+	  if (convert) {
+	       u = chunk_read_array_fp(ch,readpos,
+				       readsamples,
+				       (sample_t *)buf,dither_mode,&clipcount);
+	       u = u * full_size;
+	  } else {
+	       u = chunk_read_array(ch,readpos,readsamples * full_size,
+				    buf,dither_mode,&clipcount);
+	  }
+
+	  g_assert((u / full_size) == readsamples);
+
 	  if (!u) {
 	       chunk_close(ch);
 	       g_free(c);
 	       status_bar_end_progress(bar);
 	       return TRUE;
 	  }
+
 	  d = buf;
-	  for (x=0; x<u; x+=proc_size) {
+
+	  if(reverse) {
+	       x = u - proc_size;
+	  } else {
+	       x = 0;
+	  }
+
+	  while((x >= 0) && (x < u)) {
 	       if (proc(d+x,proc_size,chunk)) {
 		    chunk_close(ch);
 		    g_free(c);
 		    status_bar_end_progress(bar);
 		    return TRUE;
 	       }
+	       if(reverse) {
+		    x -= proc_size;
+	       } else {
+		    x += proc_size;
+	       }
+	  }
+
+	  samplesleft -= readsamples;
+	  if(reverse) {
+	       samplepos -= readsamples;
+	       if(samplepos < 0) {
+		    readsamples = readsamples + samplepos;
+		    samplepos = 0;
+	       }
+	  } else {
+	       samplepos += readsamples;
+	  }
+	  if(samplesleft < readsamples) {
+	       readsamples = samplesleft;
 	  }
-	  samplesleft -= u / full_size;
-	  samplepos += u / full_size;
 	  if (status_bar_progress(bar, u / full_size)) {
 	       chunk_close(ch);
 	       g_free(c);
@@ -520,84 +616,39 @@ gboolean chunk_parse(Chunk *chunk, chunk_parse_proc proc,
      chunk_close(ch);
      g_free(c);
      status_bar_end_progress(bar);
+     clipwarn(clipcount,FALSE);
      return FALSE;
 }
 
-static gboolean chunk_onechannel_proc(void *in, guint sample_size, 
-				      chunk_writeout_func out_func, 
-				      WriteoutID id, Dataformat *format,
-				      Dataformat *outformat)
-{
-     static int channel_count = 0;
-     static sample_t mixup = 0.0;
-     sample_t s = *((sample_t *)in);
-     mixup += s;
-     channel_count++;
-     if (channel_count == format->channels) {
-	  s = mixup / (sample_t)channel_count;
-	  channel_count = 0;
-	  mixup = 0.0;
-	  return out_func(id,&s,sizeof(s));
-     } else 
-	  return FALSE;
-}
-
 Chunk *chunk_onechannel(Chunk *chunk, int dither_mode, StatusBar *bar)
 {
-     Dataformat fmt;
-     memcpy(&fmt,&(chunk->format),sizeof(Dataformat));
-     fmt.channels = 1;
-     fmt.samplebytes = fmt.samplesize;
-     return chunk_filter_tofmt(chunk,chunk_onechannel_proc,NULL,
-			       CHUNK_FILTER_ONE,TRUE,&fmt,dither_mode,bar,
-			       NULL);
-}
-
-static gint channel_to_copy;
-
-static gboolean chunk_copy_channel_proc(void *in, guint sample_size, 
-					chunk_writeout_func out_func, 
-					WriteoutID id,
-					Dataformat *informat, 
-					Dataformat *outformat)
-{
-     static gint channel_count=0;
-     gint i;
-     i = channel_count;
-     channel_count ++;
-     if (channel_count == informat->channels) channel_count = 0;
-     if (out_func(id,in,sample_size)) return TRUE;
-     if (channel_to_copy==i) return out_func(id,in,sample_size);
-     else return FALSE;
-}
-
-static gboolean chunk_copy_channel_mono_proc(void *in, guint sample_size,
-					     chunk_writeout_func out_func, 
-					     WriteoutID id,
-					     Dataformat *informat, 
-					     Dataformat *outformat)
-{
-     return (out_func(id,in,sample_size) || out_func(id,in,sample_size));
+     gboolean *map;
+     int i;
+     Chunk *r;
+     map = g_malloc(chunk->format.channels * 1 * sizeof(gboolean));
+     for (i=0; i<chunk->format.channels; i++)
+	  map[i] = TRUE;
+     r = chunk_remap_channels(chunk,1,map,dither_mode,bar);
+     g_free(map);
+     return r;
 }
 
 Chunk *chunk_copy_channel(Chunk *chunk, gint channel, int dither_mode, 
 			  StatusBar *bar)
 {
-     Chunk *c;
-     Dataformat fmt;
-     memcpy(&fmt,&(chunk->format),sizeof(Dataformat));
-     fmt.channels ++;
-     fmt.samplebytes = fmt.channels * fmt.samplesize;
-     channel_to_copy = channel;
-     if (chunk->format.channels == 1) 
-	  c = chunk_filter_tofmt(chunk,chunk_copy_channel_mono_proc,NULL,
-				 CHUNK_FILTER_ONE,FALSE,&fmt,dither_mode,bar,
-				 NULL);
-     else 
-	  c = chunk_filter_tofmt(chunk,chunk_copy_channel_proc,NULL,
-				 CHUNK_FILTER_ONE,FALSE,&fmt,dither_mode, 
-				 bar,NULL);
-     return c;
+     Chunk *r;
+     gboolean *map;
+     int i;
+
+     map = g_malloc0(chunk->format.channels * (chunk->format.channels+1) * 
+		     sizeof(gboolean));
+     for (i=0; i<chunk->format.channels; i++)
+	  map[i*(chunk->format.channels+1) + i] = TRUE;
+     map[channel*(chunk->format.channels+1) + (chunk->format.channels)] = TRUE;
+     r = chunk_remap_channels(chunk,chunk->format.channels+1,map,dither_mode,
+			      bar);
+     g_free(map);
+     return r;
 }
 
 
@@ -606,7 +657,7 @@ Chunk *chunk_mix(Chunk *c1, Chunk *c2, int dither_mode, StatusBar *bar)
      #define BUFLEN 256
      off_t u,mixlen;
      guint i,chn,x1,x2,x;
-     guint clipcount = 0;
+     off_t clipcount = 0;
      sample_t buf1[BUFLEN],buf2[BUFLEN],buf3[BUFLEN];
      sample_t s,pmax;
      gchar *str;
@@ -639,8 +690,10 @@ Chunk *chunk_mix(Chunk *c1, Chunk *c2, int dither_mode, StatusBar *bar)
 
      for (u=0; u<mixlen; u+=x) {
 
-	  x1 = chunk_read_array_fp(ch1,u,BUFLEN/chn,buf1,dither_mode);
-	  x2 = chunk_read_array_fp(ch2,u,BUFLEN/chn,buf2,dither_mode);
+	  x1 = chunk_read_array_fp(ch1,u,BUFLEN/chn,buf1,dither_mode,
+				   &clipcount);
+	  x2 = chunk_read_array_fp(ch2,u,BUFLEN/chn,buf2,dither_mode,
+				   &clipcount);
 	  if (x1 == 0 || x2 == 0) {
 	       tempfile_abort(tmp);
 	       chunk_close(ch1);
@@ -687,8 +740,9 @@ Chunk *chunk_mix(Chunk *c1, Chunk *c2, int dither_mode, StatusBar *bar)
 
      /* Warn if clipping occured */
      if (clipcount > 0) {
+	  if (clipcount > 1000000000) clipcount = 1000000000;
 	  str = g_strdup_printf(_("The mixed result was clipped %d times."),
-				clipcount);
+				(int)clipcount);
 	  user_warning(str);
 	  g_free(str);
      }
@@ -706,6 +760,139 @@ Chunk *chunk_mix(Chunk *c1, Chunk *c2, int dither_mode, StatusBar *bar)
      return d;
 }
 
+static struct {
+     sample_t *buf;
+     int bufsize;
+     /* (outchannels x (inchannels+1))-matrix */
+     /* Each row has a list of input channel numbers, ending with -1 */
+     int *map; 
+} chunk_remap_channels_data;
+
+static gboolean chunk_remap_channels_mixmode_proc(void *in, 
+						  guint sample_size,
+						  chunk_writeout_func out_func,
+						  WriteoutID id,
+						  Dataformat *informat,
+						  Dataformat *outformat)
+{
+     int samps,i,j,k,l;
+     sample_t *iptr, *optr, s;
+     samps = sample_size / informat->samplebytes;
+
+     /* Make sure we have room in the buffer */
+     if (samps * outformat->samplebytes > chunk_remap_channels_data.bufsize) {
+	  g_free(chunk_remap_channels_data.buf);
+	  chunk_remap_channels_data.buf = 
+	       g_malloc(samps*outformat->samplebytes);
+	  chunk_remap_channels_data.bufsize = samps*outformat->samplebytes;
+     }
+
+     /* Process data */
+     iptr = (sample_t *)in;
+     optr = chunk_remap_channels_data.buf;
+
+     for (i=0; i<samps; i++) {
+	  for (j=0; j<outformat->channels; j++) {
+	       s = 0.0;
+	       for (k=(j*(informat->channels+1)); 1; k++) {
+		    l = chunk_remap_channels_data.map[k];
+		    if (l < 0) break;
+		    s += iptr[l];
+	       }
+	       optr[j] = s;
+	  }
+	  iptr += informat->channels;
+	  optr += outformat->channels;
+     }
+
+     return out_func(id, chunk_remap_channels_data.buf, 
+		     samps*outformat->samplebytes);     
+}
+
+static gboolean chunk_remap_channels_rawmode_proc(void *in, 
+						  guint sample_size,
+						  chunk_writeout_func out_func,
+						  WriteoutID id,
+						  Dataformat *informat,
+						  Dataformat *outformat)
+{
+     int samps,ssize=informat->samplesize,i,j,k;
+     char *iptr, *optr;
+
+     /* Make sure we have room in buffer */
+     samps = sample_size / informat->samplebytes;
+     if (samps * outformat->samplebytes > chunk_remap_channels_data.bufsize) {
+	  g_free(chunk_remap_channels_data.buf);
+	  chunk_remap_channels_data.buf = 
+	       g_malloc(samps*outformat->samplebytes);
+	  chunk_remap_channels_data.bufsize = samps*outformat->samplebytes;
+     }
+
+     iptr = (char *)in;
+     optr = (char *)chunk_remap_channels_data.buf;
+
+     /* This could be optimized further (replace memcpy) */
+
+     for (i=0; i<samps; i++) {
+	  for (j=0; j<outformat->channels; j++) {
+	       k = chunk_remap_channels_data.map[j*(informat->channels+1)];
+	       if (k >= 0)
+		    memcpy(optr, iptr+k*ssize,ssize);
+	       else
+		    memset(optr, 0, ssize);
+	       optr += ssize;
+	  }
+	  iptr += informat->samplebytes;
+     }
+
+     return out_func(id, chunk_remap_channels_data.buf, 
+		     samps*outformat->samplebytes);
+}
+						  
+Chunk *chunk_remap_channels(Chunk *chunk, int channels_out, gboolean *map, 
+			    int dither_mode, StatusBar *bar)
+{
+     gboolean mixmode;
+     int i,j,k;
+     int channels_in = chunk->format.channels;
+     Dataformat tofmt;
+     Chunk *r;
+     
+     memcpy(&tofmt,&(chunk->format),sizeof(Dataformat));
+     tofmt.channels = channels_out;
+     tofmt.samplebytes = tofmt.channels * tofmt.samplesize;
+
+     chunk_remap_channels_data.buf = NULL;
+     chunk_remap_channels_data.bufsize = 0;
+
+     /* Generate the map */
+     mixmode = FALSE;
+     chunk_remap_channels_data.map = g_malloc((channels_in+1)*channels_out*
+					      sizeof(int));
+     for (i=0; i<channels_out; i++) {
+	  k = 0;
+	  for (j=0; j<channels_in; j++) 
+	       if (map[j*channels_out + i])
+		    chunk_remap_channels_data.map[i*(channels_in+1)+(k++)] = j;
+	  chunk_remap_channels_data.map[i*(channels_in+1) + k] = -1;
+	  if (k > 1) mixmode = TRUE;
+     }
+
+     if (mixmode) {
+	  r = chunk_filter_tofmt(chunk,chunk_remap_channels_mixmode_proc,
+				 NULL,CHUNK_FILTER_MANY,TRUE,&tofmt,
+				 dither_mode,bar,_("Mixing channels"));
+     } else {
+	  r = chunk_filter_tofmt(chunk,chunk_remap_channels_rawmode_proc,
+				 NULL,CHUNK_FILTER_MANY,FALSE,&tofmt,
+				 dither_mode,bar,_("Mapping channels"));
+	  
+     }
+     
+     g_free(chunk_remap_channels_data.buf);
+     g_free(chunk_remap_channels_data.map);
+     return r;
+}
 
 void chunk_foreach(GFunc func, gpointer user_data)
 {
@@ -796,7 +983,8 @@ gboolean chunk_read_fp(ChunkHandle *handle, off_t sampleno, sample_t *buffer,
 }
 
 guint chunk_read_array(ChunkHandle *handle, off_t sampleno, 
-		       guint size, void *buffer, int dither_mode)
+		       guint size, void *buffer, int dither_mode, 
+		       off_t *clipcount)
 {
      GList *l;     
      DataPart *dp;
@@ -813,7 +1001,7 @@ guint chunk_read_array(ChunkHandle *handle, off_t sampleno,
 	       /* Read data. */
 	       i = datasource_read_array(dp->ds,
 					 dp->position + sampleno,
-					 j,buffer,dither_mode);
+					 j,buffer,dither_mode,clipcount);
 	       if (i == 0) return 0;
 	       r+=i;
 	       sampleno = 0;
@@ -827,7 +1015,8 @@ guint chunk_read_array(ChunkHandle *handle, off_t sampleno,
 }
 
 guint chunk_read_array_fp(ChunkHandle *handle, off_t sampleno, 
-			  guint samples, sample_t *buffer, int dither_mode)
+			  guint samples, sample_t *buffer, int dither_mode,
+			  off_t *clipcount)
 {
      GList *l;     
      DataPart *dp;
@@ -844,7 +1033,7 @@ guint chunk_read_array_fp(ChunkHandle *handle, off_t sampleno,
 		    j = samples;
 	       i = datasource_read_array_fp(dp->ds,
 					    dp->position+sampleno,
-					    j,buffer,dither_mode);
+					    j,buffer,dither_mode,clipcount);
 	       g_assert(i <= samples);
 	       g_assert(i <= dp->length - sampleno);
 	       if (i==0) return 0;
@@ -1067,7 +1256,7 @@ static gboolean chunk_peak_level_proc(void *sample, gint sample_size,
      s = (sample_t)fabs((double)(*((sample_t *)sample)));
      if (s > chunk_peak_level_max) {
 	  chunk_peak_level_max = s;
-	  if (s == 1.0) return TRUE;
+	  if (s == 1.0 && chunk->format.type == DATAFORMAT_PCM) return TRUE;
      }
      return FALSE;
 }
@@ -1076,10 +1265,144 @@ sample_t chunk_peak_level(Chunk *c, StatusBar *bar)
 {
      chunk_peak_level_max = 0.0;
      if (chunk_parse(c,chunk_peak_level_proc,FALSE,TRUE,DITHER_NONE,bar,
-		     _("Calculating peak level"))) return -1.0;
+		     _("Calculating peak level"),0,FALSE)) return -1.0;
      return chunk_peak_level_max;
 }
 
+/* BEGIN zero-crossing search added by Forest Bond */
+
+static off_t chunk_zero_crossing_seen_samples;
+static sample_t *chunk_zero_crossing_saved_sample = NULL;
+
+static gboolean chunk_signal_crossed_zero(sample_t prev, sample_t current)
+{
+     if((current >= 0.0) && (prev <= 0.0)) {
+          return TRUE;
+     } else if((current <= 0.0) && (prev >= 0.0)) {
+          return TRUE;
+     }
+     return FALSE;
+}
+
+static void chunk_zero_crossing_init(gint sample_size) {
+     if(chunk_zero_crossing_saved_sample != NULL) {
+          g_free(chunk_zero_crossing_saved_sample);
+          chunk_zero_crossing_saved_sample = NULL;
+     }
+     chunk_zero_crossing_saved_sample = g_malloc(sample_size);
+}
+
+static void chunk_zero_crossing_cleanup() {
+     if(chunk_zero_crossing_saved_sample != NULL) {
+          g_free(chunk_zero_crossing_saved_sample);
+          chunk_zero_crossing_saved_sample = NULL;
+     }
+}
+
+static void chunk_zero_crossing_save_sample(sample_t *sample,
+  gint sample_size) {
+     memcpy(chunk_zero_crossing_saved_sample, sample, sample_size);
+}
+
+static gboolean chunk_zero_crossing_any_proc(void *sample, gint sample_size, 
+				      Chunk *chunk)
+{
+     sample_t *start = (sample_t *)sample;
+     sample_t *end = start + (sample_size/sizeof(sample_t));
+     sample_t *current = start;
+     sample_t *prev = chunk_zero_crossing_saved_sample;
+
+     if(chunk_zero_crossing_seen_samples < 1) {
+          chunk_zero_crossing_init(sample_size);
+          chunk_zero_crossing_save_sample(start, sample_size);
+          chunk_zero_crossing_seen_samples++;
+          return FALSE;
+     }
+
+     while(current < end) {
+          if(chunk_signal_crossed_zero(*prev, *current)) {
+               return TRUE;
+          }
+          current++;
+          prev++;
+     }
+
+     chunk_zero_crossing_save_sample(start, sample_size);
+     chunk_zero_crossing_seen_samples++;
+     return FALSE;
+}
+
+off_t chunk_zero_crossing_any_forward(Chunk *c, StatusBar *bar,off_t samplepos)
+{
+     chunk_zero_crossing_seen_samples = 0;
+     chunk_parse(c,chunk_zero_crossing_any_proc,TRUE,TRUE,DITHER_NONE,bar,
+		     _("Finding zero-crossing"), samplepos, FALSE);
+
+     chunk_zero_crossing_cleanup();
+     return samplepos + chunk_zero_crossing_seen_samples;
+}
+
+off_t chunk_zero_crossing_any_reverse(Chunk *c, StatusBar *bar,off_t samplepos)
+{
+     chunk_zero_crossing_seen_samples = 0;
+     chunk_parse(c,chunk_zero_crossing_any_proc,TRUE,TRUE,DITHER_NONE,bar,
+		     _("Finding zero-crossing"), samplepos, TRUE);
+
+     chunk_zero_crossing_cleanup();
+     return samplepos - chunk_zero_crossing_seen_samples;
+}
+
+static gboolean chunk_zero_crossing_all_proc(void *sample, gint sample_size, 
+				      Chunk *chunk)
+{
+     sample_t *start = (sample_t *)sample;
+     sample_t *end = start + (sample_size/sizeof(sample_t));
+     sample_t *current = start;
+     sample_t *prev = chunk_zero_crossing_saved_sample;
+
+     if(chunk_zero_crossing_seen_samples < 1) {
+          chunk_zero_crossing_init(sample_size);
+          chunk_zero_crossing_save_sample(start, sample_size);
+          chunk_zero_crossing_seen_samples++;
+          return FALSE;
+     }
+
+     while(current < end) {
+
+          if(! chunk_signal_crossed_zero(*prev, *current)) {
+               chunk_zero_crossing_save_sample(start, sample_size);
+               chunk_zero_crossing_seen_samples++;
+               return FALSE;
+          }
+          current++;
+          prev++;
+     }
+     return TRUE;
+}
+
+off_t chunk_zero_crossing_all_forward(Chunk *c, StatusBar *bar,off_t samplepos)
+{
+     chunk_zero_crossing_seen_samples = 0;
+     chunk_parse(c,chunk_zero_crossing_all_proc,TRUE,TRUE,DITHER_NONE,bar,
+		     _("Finding zero-crossing"), samplepos, FALSE);
+
+     chunk_zero_crossing_cleanup();
+     return samplepos + chunk_zero_crossing_seen_samples;
+}
+
+off_t chunk_zero_crossing_all_reverse(Chunk *c, StatusBar *bar,off_t samplepos)
+{
+     chunk_zero_crossing_seen_samples = 0;
+     chunk_parse(c,chunk_zero_crossing_all_proc,TRUE,TRUE,DITHER_NONE,bar,
+		     _("Finding zero-crossing"), samplepos, TRUE);
+
+     chunk_zero_crossing_cleanup();
+
+     return samplepos - chunk_zero_crossing_seen_samples;
+}
+
+/* END zero-crossing search */
+
 static sample_t chunk_amplify_factor;
 
 static gboolean chunk_amplify_proc(void *in, guint sample_size,
@@ -1313,3 +1636,23 @@ Chunk *chunk_convert(Chunk *chunk, Dataformat *new_format,
      if (c != NULL) gtk_object_sink(GTK_OBJECT(c));
      return d;
 }
+
+gboolean clipwarn(off_t clipcount, gboolean maycancel)
+{
+     gboolean b = FALSE;
+     gchar *c;
+     if (clipcount > 0) {
+	  if (clipcount > 1000000000)
+	       clipcount = 1000000000;
+	  c = g_strdup_printf(_("The input was clipped %d times during "
+				"processing."),(int)clipcount);
+	  if (maycancel)
+	       b = (user_message(c,UM_OKCANCEL) != MR_OK);
+	  else
+	       user_warning(c);
+	  
+	  g_free(c);
+     }
+
+     return b;
+}
