@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002 2003 2004 2005, Magnus Hjorth
+ * Copyright (C) 2002 2003 2004 2005 2006, Magnus Hjorth
  *
  * This file is part of mhWaveEdit.
  *
@@ -349,6 +349,7 @@ void convert_array(void *indata, Dataformat *indata_format,
 		   guint count, int dither_mode)
 {     
      int i;
+     char *c;
      if (dataformat_samples_equal(indata_format,outdata_format)) {
 	  memcpy(outdata,indata,count*indata_format->samplesize);
      } else if (indata_format->type == DATAFORMAT_PCM) {
@@ -356,7 +357,6 @@ void convert_array(void *indata, Dataformat *indata_format,
 	       /* PCM -> PCM conversion */
 	       if (outdata_format->samplesize > indata_format->samplesize)
 		    dither_mode = DITHER_NONE;
-	       char *c;
 	       c = g_malloc(count * sizeof(sample_t));	       
 	       convert_array(indata,indata_format,c,&dataformat_sample_t,
 			     count,dither_mode);
@@ -380,6 +380,7 @@ void convert_array(void *indata, Dataformat *indata_format,
 	       (outdata_format->bigendian?2:0) + 
 	       (indata_format->samplesize/sizeof(double));
 	  g_assert(i < ARRAY_LENGTH(fp_pcm_functions));
+	  g_assert(dither_mode != DITHER_UNSPEC);
 	  if (dither_mode != DITHER_NONE) {
 	       if (indata_format->samplesize == sizeof(float))
 		    dither_convert_float(indata,outdata,count,
@@ -407,6 +408,15 @@ void convert_array(void *indata, Dataformat *indata_format,
      }
 }
 
+gint unnormalized_count(sample_t *buf, gint count)
+{
+     gint i,c=0;
+     for (i=0; i<count; i++)
+	  if (buf[i] > 1.0 || buf[i] < -1.0)
+	       c++;
+     return c;
+}
+
 static void print_format(Dataformat *fmt)
 {
      if (fmt->type == DATAFORMAT_FLOAT) {
