@@ -516,9 +516,9 @@ gboolean document_apply(Document *d, chunk_filter_proc proc,
 		* If the result is shorter, report the difference as a 
 		* removal at the new selection's right endpoint */
 	       if (rlen > plen)
-		    document_update(d, r, d->selstart+plen, rlen-plen);
+		    document_update(d, c, d->selstart+plen, rlen-plen);
 	       else
-		    document_update(d, r, d->selstart+rlen, rlen-plen);
+		    document_update(d, c, d->selstart+rlen, rlen-plen);
 	       return FALSE;
 	  } else
 	       return TRUE;
@@ -535,14 +535,14 @@ void document_parse(Document *d, chunk_parse_proc proc,
 	 (d->selstart==0 && d->selend >= d->chunk->length)) {
 	  /* procstart(w); */
 	  chunk_parse(d->chunk,proc,allchannels,convert,dither_editing,
-		      d->bar,title);
+		      d->bar,title,0,FALSE);
 	  /* procend(w); */
      } else {
 	  c = chunk_get_part(d->chunk,d->selstart,
 			     d->selend - d->selstart);
 	  /* procstart(w); */
 	  chunk_parse(c,proc,allchannels,convert,dither_editing,
-		      d->bar,title);
+		      d->bar,title,0,FALSE);
 	  /* procend(w); */
 	  gtk_object_sink(GTK_OBJECT(c));
      }
@@ -647,6 +647,24 @@ void document_set_cursor(Document *d, off_t cursorpos)
      document_set_cursor_main(d,cursorpos,FALSE);
 }
 
+off_t document_nudge_cursor(Document *d, off_t delta)
+{
+     off_t newpos;
+
+     newpos = d->cursorpos + delta;
+     if(newpos > d->chunk->length)
+     {
+          delta = (d->chunk->length) - (d->cursorpos);
+     } else if(newpos < 0) {
+          delta = - (d->cursorpos);
+     }
+
+     newpos = d->cursorpos + delta;
+     document_set_cursor(d,newpos);
+
+     return delta;
+}
+
 void document_set_followmode(Document *d, gboolean mode)
 {
      d->followmode = mode;
