@@ -61,37 +61,37 @@ static GList *file_types = NULL;
 
 static gboolean wav_check(gchar *filename);
 static Chunk *wav_load(gchar *filename, int dither_mode, StatusBar *bar);
-static gboolean wav_save(Chunk *chunk, gchar *filename, gpointer settings,
-			 struct file_type *type, int dither_mode, 
-			 StatusBar *bar,gboolean *fatal);
+static gint wav_save(Chunk *chunk, gchar *filename, gpointer settings,
+		     struct file_type *type, int dither_mode, 
+		     StatusBar *bar,gboolean *fatal);
 
 #ifdef HAVE_LIBSNDFILE
 static gboolean sndfile_check(gchar *filename);
 static Chunk *sndfile_load(gchar *filename, int dither_mode, StatusBar *bar);
-static gboolean sndfile_save(Chunk *chunk, gchar *filename, gpointer settings,
-			     struct file_type *type, int dither_mode, 
-			     StatusBar *bar, gboolean *fatal);
+static gint sndfile_save(Chunk *chunk, gchar *filename, gpointer settings,
+			 struct file_type *type, int dither_mode, 
+			 StatusBar *bar, gboolean *fatal);
 static gboolean sndfile_save_main(Chunk *chunk, gchar *filename, 
 				  int format, int dither_mode, StatusBar *bar,
 				  gboolean *fatal);
 #endif
 
 static Chunk *raw_load(gchar *filename, int dither_mode, StatusBar *bar);
-static gboolean raw_save(Chunk *chunk, gchar *filename, gpointer settings,
-			 struct file_type *type, int dither_mode,
-			 StatusBar *bar, gboolean *fatal);
+static gint raw_save(Chunk *chunk, gchar *filename, gpointer settings,
+		     struct file_type *type, int dither_mode,
+		     StatusBar *bar, gboolean *fatal);
 
 static Chunk *ogg_load(gchar *filename, int dither_mode, StatusBar *bar);
-static gboolean ogg_save(Chunk *chunk, gchar *filename, gpointer settings,
-			 struct file_type *type,
-			 int dither_mode, StatusBar *bar, gboolean *fatal);
+static gint ogg_save(Chunk *chunk, gchar *filename, gpointer settings,
+		     struct file_type *type,
+		     int dither_mode, StatusBar *bar, gboolean *fatal);
 
 
 static gpointer mp3_get_settings(void);
 static Chunk *mp3_load(gchar *filename, int dither_mode, StatusBar *bar);
-static gboolean mp3_save(Chunk *chunk, gchar *filename, gpointer settings,
-			 struct file_type *type,
-			 int dither_mode, StatusBar *bar, gboolean *fatal);
+static gint mp3_save(Chunk *chunk, gchar *filename, gpointer settings,
+		     struct file_type *type,
+		     int dither_mode, StatusBar *bar, gboolean *fatal);
 
 static Chunk *try_mplayer(gchar *filename, int dither_mode, StatusBar *bar);
 
@@ -121,9 +121,9 @@ static struct file_type *register_file_type
 (gchar *name, gchar *ext, gboolean lossy,
  gboolean (*typecheck)(gchar *filename), 
  Chunk *(*load)(gchar *filename, int dither_mode, StatusBar *bar),
- gboolean (*save)(Chunk *chunk, gchar *filename,gpointer settings,
-		  struct file_type *type,int dither_mode,
-		  StatusBar *bar,gboolean *fatal),
+ gint (*save)(Chunk *chunk, gchar *filename,gpointer settings,
+	      struct file_type *type,int dither_mode,
+	      StatusBar *bar,gboolean *fatal),
  int extra_data)
 {
      struct file_type *t;
@@ -530,7 +530,10 @@ static gboolean wav_save(Chunk *chunk, char *filename, gpointer settings,
      Datasource *ds;
      DataPart *dp;
      Dataformat *fmt;
+     gboolean b;
+
      fmt = &(chunk->format);
+     
 
      /* Check if the format is OK */     
 
@@ -603,13 +606,15 @@ static gboolean wav_save(Chunk *chunk, char *filename, gpointer settings,
 	       e_fclose(f);
 	       return TRUE;
 	  }
-	  if (chunk_dump(chunk,f,FALSE,dither_mode,bar)) {
+	  b = chunk_dump(chunk,f,FALSE,dither_mode,bar);
+	  if (b) {
 	       e_fclose_remove(f);
 	       return TRUE;
 	  }
 	  e_fclose ( f );
+	  return FALSE;
      }    
-     return FALSE;
+     return 0;
 }
 
 /* RAW */
@@ -633,20 +638,20 @@ static Chunk *raw_load(gchar *filename, int dither_mode, StatusBar *bar)
      return chunk_new_from_datasource(ds);
 }
 
-static gboolean raw_save(Chunk *chunk, gchar *filename, gpointer settings,
+static gint raw_save(Chunk *chunk, gchar *filename, gpointer settings,
 			 struct file_type *type, int dither_mode, 
 			 StatusBar *bar, gboolean *fatal)
 {
      EFILE *f;
+     gint i;
+
      f = e_fopen(filename,EFILE_WRITE);
      if (!f) return TRUE;
-     if (chunk_dump(chunk,f,IS_BIGENDIAN,dither_mode,bar)) {
-          e_fclose(f);
-	  *fatal = TRUE;
-          return TRUE;
-     }
+    
+     i = chunk_dump(chunk,f,IS_BIGENDIAN,dither_mode,bar);
+     if (i < 0) *fatal = TRUE;
      e_fclose(f);
-     return FALSE;
+     return i;
 }
 
 /* SNDFILE */
@@ -721,7 +726,7 @@ static Chunk *sndfile_load(gchar *filename, int dither_mode, StatusBar *bar)
      while (ds->data.sndfile.pos < ds->length) {
 
 	  st = datasource_read_array(ds,ds->data.sndfile.pos,rawbuf_size,
-				     rawbuf, dither_mode);
+				     rawbuf, dither_mode, NULL);
 	  
 	  if (st == 0 || tempfile_write(tmp,rawbuf,st) || 
 	      status_bar_progress(bar, st/f.samplebytes)) {
@@ -802,16 +807,17 @@ static gboolean sndfile_save_main(Chunk *chunk, gchar *filename,
      SF_INFO info;
      sample_t *samplebuf;
      ChunkHandle *ch;
+     off_t clipcount = 0;
 
      if (find_nearest_sndfile_format(&(chunk->format),format,&info,filename)) 
-	  return TRUE;
+	  return -1;
      
      s = sf_open(filename, SFM_WRITE, &info);
      if (!s) {
           c = g_strdup_printf(_("Failed to open '%s'!"),filename);
           user_error(c);
           g_free(c);
-          return TRUE;
+          return -1;
      }
 
      sf_command ( s, SFC_SET_NORM_FLOAT, NULL, SF_TRUE );
@@ -820,7 +826,7 @@ static gboolean sndfile_save_main(Chunk *chunk, gchar *filename,
      ch = chunk_open(chunk);
      if (ch == NULL) {
 	  sf_close(s);
-	  return TRUE;
+	  return -1;
      }
 
      /* We could speed this up a LOT esp. on output to PCM files. */
@@ -828,14 +834,15 @@ static gboolean sndfile_save_main(Chunk *chunk, gchar *filename,
 
      for (i=0; i<chunk->length; i+=n) {
 	  /* FIXME: Use chunk_read_array for FP chunks */
-	  n = chunk_read_array_fp(ch,i,1024,samplebuf,dither_mode);
+	  n = chunk_read_array_fp(ch,i,1024,samplebuf,dither_mode,&clipcount);
 	  if (!n) {
 	       chunk_close(ch);
 	       sf_close(s);
 	       g_free(samplebuf);
 	       *fatal = TRUE;
-	       return TRUE;
+	       return -1;
 	  }
+	  clipcount += unnormalized_count(samplebuf,n*chunk->format.channels);
 	  if (sf_writef_sample_t(s,samplebuf,n) != n) {
 	       c = g_strdup_printf(_("Failed to write to '%s'!"),filename);
 	       user_error(c);
@@ -844,27 +851,26 @@ static gboolean sndfile_save_main(Chunk *chunk, gchar *filename,
 	       sf_close(s);
 	       g_free(samplebuf);
 	       *fatal = TRUE;
-	       return TRUE;  
+	       return -1;  
 	  }
 	  if (status_bar_progress(bar, n*chunk->format.samplebytes)) {
 	       chunk_close(ch);
 	       sf_close(s);
 	       g_free(samplebuf);
 	       xunlink(filename);
-	       return TRUE;
+	       return -1;
 	  }
      }
      chunk_close(ch);
      sf_close(s);
      g_free(samplebuf);
-
-     return FALSE;
+     return clipwarn(clipcount,TRUE);
 
 }
 
-static gboolean sndfile_save(Chunk *chunk, gchar *filename, gpointer settings,
-			     struct file_type *type, int dither_mode, 
-			     StatusBar *bar, gboolean *fatal)
+static gint sndfile_save(Chunk *chunk, gchar *filename, gpointer settings,
+			 struct file_type *type, int dither_mode, 
+			 StatusBar *bar, gboolean *fatal)
 {
      return sndfile_save_main(chunk,filename,type->extra_data,dither_mode,
 			      bar,fatal);
@@ -955,6 +961,7 @@ static gboolean ogg_save(Chunk *chunk, gchar *filename, gpointer settings,
      Dataformat fmt;
      gchar *c,*d;
      gboolean b;
+     off_t clipcount = 0;
      xunlink(filename);
      d = g_strdup_printf("OUTFILE=%s",filename);     
      if (xputenv(d)) { g_free(d); return TRUE; }
@@ -972,15 +979,15 @@ static gboolean ogg_save(Chunk *chunk, gchar *filename, gpointer settings,
 			 y->format.samplesize*8, 
 			 y->format.channels, y->format.samplerate,
 			 y->format.bigendian?1:0);
-     b = pipe_dialog_send_chunk(y,c,FALSE,dither_mode,bar);
+     b = pipe_dialog_send_chunk(y,c,FALSE,dither_mode,bar,&clipcount);
      g_free(c);     
      if (x != NULL) gtk_object_sink(GTK_OBJECT(x));
      if (!xunsetenv("OUTFILE")) g_free(d);
      if (b || !file_exists(filename)) {
 	  *fatal = TRUE;
-	  return TRUE;
+	  return -1;
      }
-     return FALSE;
+     return clipwarn(clipcount,TRUE);
 }
 
 static struct {
@@ -1208,24 +1215,26 @@ static gboolean wav_regular_format(Dataformat *fmt)
      return (XOR(fmt->samplesize == 1, fmt->sign));
 }
 
-static gboolean mp3_save(Chunk *chunk, gchar *filename, gpointer settings,
-			 struct file_type *type,
-			 int dither_mode, StatusBar *bar, gboolean *fatal)
+static gint mp3_save(Chunk *chunk, gchar *filename, gpointer settings,
+		     struct file_type *type,
+		     int dither_mode, StatusBar *bar, gboolean *fatal)
 {
      Chunk *x;
      Dataformat fmt;
      gchar *c;
      gboolean b;
+     off_t clipcount = 0;
      xunlink(filename);
      c = g_strdup_printf("OUTFILE=%s",filename);
-     if (xputenv(c)) { g_free(c); return TRUE; }
+     if (xputenv(c)) { g_free(c); return -1; }
      c = g_strdup_printf("LAMEFLAGS=%s",(settings == NULL) ? 
 			 "--preset standard" : (gchar *)settings);
-     if (xputenv(c)) { g_free(c); return TRUE; }
+     if (xputenv(c)) { g_free(c); return -1; }
      if (wav_regular_format(&(chunk->format)))
 	 b = pipe_dialog_send_chunk(chunk,
 				    "lame --silent $LAMEFLAGS - "
-				    "\"$OUTFILE\"",TRUE,dither_mode,bar);
+				    "\"$OUTFILE\"",TRUE,dither_mode,bar,
+				    &clipcount);
      else {
 	  memcpy(&fmt,&(chunk->format),sizeof(Dataformat));
 	  if (fmt.type == DATAFORMAT_FLOAT || fmt.samplesize == 3) {
@@ -1240,16 +1249,17 @@ static gboolean mp3_save(Chunk *chunk, gchar *filename, gpointer settings,
 	  else {
 	       b = pipe_dialog_send_chunk(x,
 					  "lame --silent --preset standard - "
-					  "\"$OUTFILE\"",TRUE,dither_mode,bar);
+					  "\"$OUTFILE\"",TRUE,dither_mode,bar,
+					  &clipcount);
 	       gtk_object_sink(GTK_OBJECT(x));
 	  }
      }
      if (!xunsetenv("OUTFILE")) g_free(c);
      if (b || !file_exists(filename)) {
 	  *fatal = TRUE;
-	  return TRUE;
+	  return -1;
      }
-     return FALSE;
+     return clipwarn(clipcount,TRUE);
 }
 
 
@@ -1260,6 +1270,10 @@ static Chunk *try_mplayer(gchar *filename, int dither_mode, StatusBar *bar)
      gchar *c,*d;
      char *tempname;
      Chunk *x;
+     char *argv[] = { "sh", "-c", 
+		      "mplayer -quiet -noconsolecontrols "
+		      "-ao \"pcm:file=$OUTFILE\" -vc dummy -vo null "
+		      "\"$INFILE\"", NULL };
      if (!program_exists("mplayer")) return NULL;
      tempname = get_temp_filename(0);
      c = g_strdup_printf("OUTFILE=%s",tempname);
@@ -1272,10 +1286,6 @@ static Chunk *try_mplayer(gchar *filename, int dither_mode, StatusBar *bar)
 	  g_free(tempname);
 	  return NULL; 
      }
-     char *argv[] = { "sh", "-c", 
-		      "mplayer -quiet -noconsolecontrols "
-		      "-ao \"pcm:file=$OUTFILE\" -vc dummy -vo null "
-		      "\"$INFILE\"", NULL };
 
      x = run_decoder(filename,tempname,"sh",argv,dither_mode,bar);
 
