@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002 2003 2004 2005 2006, Magnus Hjorth
+ * Copyright (C) 2002 2003 2004 2005 2006 2007, Magnus Hjorth
  *
  * This file is part of mhWaveEdit.
  *
@@ -447,13 +447,15 @@ static void mainwindow_toggle_mark(Mainwindow *w, gchar *label)
 
 static void do_play(Mainwindow *w, off_t start, off_t end, gboolean loop)
 {
-     gfloat s;
+     gboolean speed_reset = FALSE;
      if (varispeed_reset_flag && 
 	 !(player_playing() && playing_document == w->doc))
-	  s = 1.0;
-     else
-	  s = player_get_speed();
-     document_play(w->doc, start, end, loop, s);
+	  speed_reset = TRUE;
+     if (speed_reset) {
+	  document_stop(w->doc,FALSE);
+	  gtk_adjustment_set_value(w->speed_adj,-1.0);
+     }
+     document_play(w->doc, start, end, loop, -w->speed_adj->value);
 }
 
 
@@ -503,10 +505,24 @@ static gint mainwindow_keypress(GtkWidget *widget, GdkEventKey *event)
 	  case GDK_9: mainwindow_toggle_mark(w,"9"); return TRUE;
 	  case GDK_Left:
 	  case GDK_KP_Left: 
-	       if (playing_document == w->doc) player_nudge(-0.5); return TRUE;
+	       if (playing_document == w->doc) 
+		    player_nudge(-0.5); 
+	       else {
+		    o = w->doc->cursorpos - w->doc->chunk->format.samplerate/2;
+		    if (o < 0) o = 0;
+		    document_set_cursor(w->doc,o);
+	       }
+	       return TRUE;
 	  case GDK_Right:
 	  case GDK_KP_Right:
-	       if (playing_document == w->doc) player_nudge(0.5); return TRUE;
+	       if (playing_document == w->doc) 
+		    player_nudge(0.5); 
+	       else {
+		    o = w->doc->cursorpos + w->doc->chunk->format.samplerate/2;
+		    if (o > w->doc->chunk->length) o = w->doc->chunk->length;
+		    document_set_cursor(w->doc,o);
+	       }
+	       return TRUE;
 	  }
      else 
 	  switch (event->keyval) {	       
@@ -520,11 +536,19 @@ static gint mainwindow_keypress(GtkWidget *widget, GdkEventKey *event)
 	  case GDK_7: mainwindow_goto_mark(w,"7"); return TRUE;
 	  case GDK_8: mainwindow_goto_mark(w,"8"); return TRUE;
 	  case GDK_9: mainwindow_goto_mark(w,"9"); return TRUE;
-	  case GDK_plus: view_zoomin(NULL,widget); return TRUE;
-	  case GDK_KP_Add: view_zoomin(NULL,widget); return TRUE;
-	  case GDK_equal: view_zoomin(NULL,widget); return TRUE;
-	  case GDK_minus: view_zoomout(NULL,widget); return TRUE;
-	  case GDK_KP_Subtract: view_zoomout(NULL,widget); return TRUE;
+	  case GDK_plus:
+	  case GDK_KP_Add:
+	  case GDK_equal:
+	       view_zoomin(NULL,widget); return TRUE;
+	  case GDK_Down:
+	  case GDK_KP_Down:
+	       document_zoom(w->doc,2.0,FALSE); return TRUE;
+	  case GDK_minus:
+	  case GDK_KP_Subtract: 
+	       view_zoomout(NULL,widget); return TRUE;
+	  case GDK_Up:
+	  case GDK_KP_Up:
+	       document_zoom(w->doc,0.5,FALSE); return TRUE;
 	  case GDK_greater: view_zoomtoselection(NULL,widget); return TRUE;
 	  case GDK_less: view_zoomall(NULL,widget); return TRUE;
 	  case GDK_comma: edit_play(NULL,widget); return TRUE;
@@ -540,12 +564,12 @@ static gint mainwindow_keypress(GtkWidget *widget, GdkEventKey *event)
 	  case GDK_Left:
 	  case GDK_KP_Left:
 	       document_scroll(w->doc, 
-			       -(w->doc->viewend - w->doc->viewstart)/4);
+			       -(w->doc->viewend - w->doc->viewstart)/MAINWINDOW_SCROLL_DELTA_RATIO);
 	       return TRUE;
 	  case GDK_Right:
 	  case GDK_KP_Right:
 	       document_scroll(w->doc,
-			       (w->doc->viewend - w->doc->viewstart)/4);
+			       (w->doc->viewend - w->doc->viewstart)/MAINWINDOW_SCROLL_DELTA_RATIO);
 	       return TRUE;
 	  case GDK_parenleft:
 	       o = 3*w->doc->chunk->format.samplerate;
@@ -902,7 +926,16 @@ static void edit_silence(GtkMenuItem *menuitem, gpointer user_data)
 static void edit_selectall(GtkMenuItem *menuitem, gpointer user_data)
 {
      Mainwindow *w = MAINWINDOW(user_data);
-     document_set_selection(w->doc,0,w->doc->chunk->length);
+     if (w->doc->selstart != 0 || w->doc->selend != w->doc->chunk->length)
+	  document_set_selection(w->doc,0,w->doc->chunk->length);
+     else
+	  document_set_selection(w->doc,0,0);
+}
+
+static void edit_selectnone(GtkMenuItem *menuitem, gpointer user_data)
+{
+     Mainwindow *w = MAINWINDOW(user_data);
+     document_set_selection(w->doc,0,0);
 }
 
 static void view_zoomin(GtkMenuItem *menuitem, gpointer user_data)
@@ -1088,6 +1121,11 @@ static void effects_splitchannel(GtkMenuItem *menuitem, gboolean user_data)
      document_update(w->doc, c, 0, 0);
 }
 
+static void effects_mapchannels(GtkMenuItem *menuitem, gpointer user_data)
+{
+     mainwindow_show_effect_dialog(MAINWINDOW(user_data),"mapchannels");
+}
+
 static void edit_stop(GtkMenuItem *menu_item, gpointer user_data)
 {
      Mainwindow *w = MAINWINDOW(user_data);
@@ -1258,7 +1296,7 @@ gtk_window_set_modal(GTK_WINDOW(a),TRUE);
      gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
      gtk_widget_show(c);
      c = gtk_label_new(_("Created by Magnus Hjorth (magnus.hjorth@home.se)\n"
-	  "Copyright 2002-2006, Magnus Hjorth"));
+	  "Copyright 2002-2007, Magnus Hjorth"));
      gtk_box_pack_start(GTK_BOX(b),c,FALSE,FALSE,0);
      gtk_widget_show(c);
      c = gtk_hseparator_new();
@@ -1399,7 +1437,8 @@ static void edit_insertsilence(GtkMenuItem *menuitem, gpointer user_data)
      gfloat f;
      off_t cp;
      Chunk *c,*nc;
-     if (user_input_float(_("Seconds of silence: "),_("Insert Silence"),0.0,&f)) 
+     if (user_input_float(_("Seconds of silence: "),_("Insert Silence"),0.0,
+			  GTK_WINDOW(w),&f)) 
 	  return;
      if (f<=0.0) return;
      c = chunk_new_silent(&(w->doc->chunk->format),f);
@@ -1428,10 +1467,11 @@ static Chunk *effects_normalize_proc(Chunk *chunk, StatusBar *bar,
 {
      sample_t s;
      Chunk *c;
+     gfloat *lp = (gfloat *)user_data;
+     sample_t level = (lp == NULL) ? 1.0 : *lp;
      s = chunk_peak_level(chunk,bar);
-     if (s <= 0.0) return NULL;
-     c = chunk_amplify(chunk,maximum_float_value(&(chunk->format))/s,
-		       dither_editing,bar);
+     if ((s < 0.0 && level >= 1.0) || s == 0.0) return NULL;
+     c = chunk_amplify(chunk,level/s,dither_editing,bar);
      return c;
 }
 
@@ -1441,34 +1481,147 @@ static void effects_normalize(GtkMenuItem *menuitem, gpointer user_data)
 		       TRUE,NULL);
 }
 
+static void effects_normalizeto(GtkMenuItem *menuitem, gpointer user_data)
+{
+     gboolean b;
+     gfloat val;
+     val = inifile_get_gfloat("lastNormLevel",1.0);
+     b = user_input_float(_("Level: "),_("Normalize to..."),val,
+			  GTK_WINDOW(user_data),&val);
+     if (b) return;
+     b = document_apply_cb(MAINWINDOW(user_data)->doc,effects_normalize_proc,
+			   TRUE,&val);
+     if (b) return;
+     inifile_set_gfloat("lastNormLevel",val);    
+}
+
+
 static void effects_pipe(GtkMenuItem *menuitem, gpointer user_data)
 {
      mainwindow_show_effect_dialog(MAINWINDOW(user_data),"pipe");
 }
 
-static void cursor_tobeginning(GtkMenuItem *menuitem, gpointer user_data)
+static void cursor_moveto_beginning(GtkMenuItem *menuitem, gpointer user_data)
 {
      document_set_cursor(MAINWINDOW(user_data)->doc,0);
 }
 
-static void cursor_toend(GtkMenuItem *menuitem, gpointer user_data)
+static void cursor_moveto_end(GtkMenuItem *menuitem, gpointer user_data)
 {
      Mainwindow *w = MAINWINDOW(user_data);
      document_set_cursor(w->doc,w->doc->chunk->length);
 }
 
-static void cursor_toselstart(GtkMenuItem *menuitem, gpointer user_data)
+static void cursor_moveto_selstart(GtkMenuItem *menuitem, gpointer user_data)
 {
      Mainwindow *w = MAINWINDOW(user_data);
      document_set_cursor(w->doc,w->doc->selstart);
 }
 
-static void cursor_toselend(GtkMenuItem *menuitem, gpointer user_data)
+static void cursor_moveto_selend(GtkMenuItem *menuitem, gpointer user_data)
 {
      Mainwindow *w = MAINWINDOW(user_data);
      document_set_cursor(w->doc,w->doc->selend);
 }
 
+static void cursor_move_left(GtkMenuItem *menuitem, gpointer user_data)
+{
+     off_t delta;
+     Mainwindow *w = MAINWINDOW(user_data);
+
+     delta =
+      - ((w->doc->viewend - w->doc->viewstart)/MAINWINDOW_NUDGE_DELTA_RATIO);
+
+     if(delta > -1) {
+          delta = -1;
+     }
+
+     document_nudge_cursor(w->doc,delta);
+}
+
+static void cursor_move_right(GtkMenuItem *menuitem, gpointer user_data)
+{
+     off_t delta;
+     Mainwindow *w = MAINWINDOW(user_data);
+
+     delta =
+      ((w->doc->viewend - w->doc->viewstart)/MAINWINDOW_NUDGE_DELTA_RATIO);
+
+     if(delta < 1) {
+          delta = 1;
+     }
+
+     document_nudge_cursor(w->doc,delta);
+}
+
+static void cursor_move_leftsample(GtkMenuItem *menuitem, gpointer user_data)
+{
+     Mainwindow *w = MAINWINDOW(user_data);
+     document_nudge_cursor(w->doc,-1);
+}
+
+static void cursor_move_rightsample(GtkMenuItem *menuitem, gpointer user_data)
+{
+     Mainwindow *w = MAINWINDOW(user_data);
+     document_nudge_cursor(w->doc,1);
+}
+
+static void cursor_findzerocrossing_leftall(GtkMenuItem *menuitem,
+ gpointer user_data)
+{
+     Mainwindow *w = MAINWINDOW(user_data);
+     off_t newpos;
+
+     newpos = chunk_zero_crossing_all_reverse(
+      w->doc->chunk,w->statusbar,w->doc->cursorpos);
+
+     if(newpos >= 0) {
+         document_set_cursor(w->doc,newpos);
+     }
+}
+
+static void cursor_findzerocrossing_rightall(GtkMenuItem *menuitem,
+ gpointer user_data)
+{
+     Mainwindow *w = MAINWINDOW(user_data);
+     off_t newpos;
+
+     newpos = chunk_zero_crossing_all_forward(
+      w->doc->chunk,w->statusbar,w->doc->cursorpos);
+
+     if(newpos >= 0) {
+         document_set_cursor(w->doc,newpos);
+     }
+}
+
+static void cursor_findzerocrossing_leftany(GtkMenuItem *menuitem,
+ gpointer user_data)
+{
+     Mainwindow *w = MAINWINDOW(user_data);
+     off_t newpos;
+
+     newpos = chunk_zero_crossing_any_reverse(
+      w->doc->chunk,w->statusbar,w->doc->cursorpos);
+
+     if(newpos >= 0) {
+          document_set_cursor(w->doc,newpos);
+     }
+}
+
+static void cursor_findzerocrossing_rightany(GtkMenuItem *menuitem,
+ gpointer user_data)
+{
+     Mainwindow *w = MAINWINDOW(user_data);
+     off_t newpos;
+
+     newpos = chunk_zero_crossing_any_forward(
+      w->doc->chunk,w->statusbar,w->doc->cursorpos);
+
+     if(newpos >= 0) {
+          document_set_cursor(w->doc,newpos);
+     }
+}
+
 static Chunk *byteswap_proc(Chunk *chunk, StatusBar *bar, 
 			    gpointer user_data)
 {
@@ -1527,6 +1680,15 @@ static gchar *translate_menu_path(const gchar *path, gpointer func_data)
     return _(path);
 }
 
+static GtkWidget *xgtk_item_factory_get_item(GtkItemFactory *itemf, 
+					     gchar *caption)
+{
+     GtkWidget *w;
+     w = gtk_item_factory_get_item(itemf,caption);
+     g_assert(w != NULL);
+     return w;
+}
+
 static GtkWidget *create_menu(Mainwindow *w)
 {
      guint i,j;
@@ -1552,9 +1714,10 @@ static GtkWidget *create_menu(Mainwindow *w)
 	  { N_("/Edit/Paste to _new"),NULL,     edit_pastetonew,0, NULL          },
 	  { N_("/Edit/Cr_op"),    NULL,         edit_crop,      0, NULL          },
 	  { N_("/Edit/_Delete"),  "<control>D", edit_delete,    0, NULL          },
-	  { N_("/Edit/Silence selection"),NULL,edit_silence,   0, NULL          },
+	  { N_("/Edit/Silence selection"),NULL,edit_silence,   0, NULL           },
 	  { N_("/Edit/sep2"),     NULL,         NULL,           0, "<Separator>" },
 	  { N_("/Edit/Select _all"),"<control>A",edit_selectall,0, NULL          },
+	  { N_("/Edit/Select none"),NULL,edit_selectnone,0, NULL                 },
 	  { N_("/Edit/sep3"),     NULL,         NULL,           0, "<Separator>" },
 	  { N_("/Edit/Clear clipboard"),NULL,   edit_clearclipboard,0,NULL       },
 	  { N_("/Edit/sep4"),     NULL,         NULL,           0, "<Separator>" },
@@ -1570,20 +1733,35 @@ static GtkWidget *create_menu(Mainwindow *w)
 	  { N_("/View/_Horizontal zoom"),NULL,  view_horizoom,  0, "<CheckItem>" },
 	  { N_("/View/_Vertical zoom"),NULL,    view_vertzoom,  0, "<CheckItem>" },
 	  { N_("/View/Sp_eed slider"),NULL,     view_speed,     0, "<CheckItem>" },
-	  { N_("/_Cursor"),       NULL,         NULL,           0, "<Branch>" },
+	  { N_("/_Cursor"),       NULL,         NULL,           0, "<Branch>"    },
 	  {N_("/Cursor/Set selection start"),"<control>Q",edit_selstartcursor,0,
 	   NULL},
 	  { N_("/Cursor/Set selection end"),"<control>W",edit_selendcursor,0,NULL},
 	  { N_("/Cursor/sep1"),   NULL,         NULL,           0, "<Separator>" },
-	  {N_("/Cursor/Move to beginning"),"<control>H",cursor_tobeginning,0,
-	   NULL},
-	  {N_("/Cursor/Move to end"),"<control>J",cursor_toend, 0, NULL          },
-	  {N_("/Cursor/Move to selection start"),"<control>K",cursor_toselstart,
-	   0,NULL},
-	  {N_("/Cursor/Move to selection end"),"<control>L",cursor_toselend,0,
+      { N_("/Cursor/Move to"),   NULL,         NULL,           0, "<Branch>" },
+	  { N_("/Cursor/Move to/Beginning"),"<control>H",cursor_moveto_beginning,0,
 	   NULL},
+	  { N_("/Cursor/Move to/End"),"<control>J",cursor_moveto_end, 0, NULL   },
+	  { N_("/Cursor/Move to/Selection start"),"<control>K",
+       cursor_moveto_selstart,0,NULL},
+	  { N_("/Cursor/Move to/Selection end"),"<control>L",
+       cursor_moveto_selend,0,NULL},
+      { N_("/Cursor/Move"),   NULL,         NULL,           0, "<Branch>" },
+	  { N_("/Cursor/Move/Left"),"H",cursor_move_left,0,NULL},
+	  { N_("/Cursor/Move/Right"),"J",cursor_move_right,0,NULL},
+	  { N_("/Cursor/Move/Left sample"),"K",cursor_move_leftsample,0,NULL},
+	  { N_("/Cursor/Move/Right sample"),"L",cursor_move_rightsample,0,NULL},
+      { N_("/Cursor/Find zero-crossing"),NULL,NULL,0,"<Branch>"},
+	  { N_("/Cursor/Find zero-crossing/Left (all channels)"),"Y",
+       cursor_findzerocrossing_leftall,0,NULL},
+	  { N_("/Cursor/Find zero-crossing/Right (all channels)"),"U",
+       cursor_findzerocrossing_rightall,0,NULL},
+	  { N_("/Cursor/Find zero-crossing/Left (any channel)"),"I",
+       cursor_findzerocrossing_leftany,0,NULL},
+	  { N_("/Cursor/Find zero-crossing/Right (any channel)"),"O",
+       cursor_findzerocrossing_rightany,0,NULL},
 	  { N_("/Cursor/sep2"),   NULL,         NULL,           0, "<Separator>" },
-	  {N_("/Cursor/Position cursor..."),"<control>G",edit_positioncursor,0,
+	  { N_("/Cursor/Position cursor..."),"<control>G",edit_positioncursor,0,
 	   NULL},
 	  { N_("/_Play"),         NULL,         NULL,           0, "<Branch>"    },
 	  { N_("/Play/_Play from cursor"),NULL, edit_play,      0, NULL          },
@@ -1591,11 +1769,12 @@ static GtkWidget *create_menu(Mainwindow *w)
 	  { N_("/Play/Play se_lection"),NULL,   edit_playselection,0,NULL        },
 	  { N_("/Play/_Stop"),    NULL,         edit_stop,      0, NULL          },
 	  { N_("/Play/sep1"),     NULL,         NULL,           0, "<Separator>" },
-	  { N_("/Play/_Record..."),  "F12",        edit_record,    0, NULL          },
+	  { N_("/Play/_Record..."),  "F12",        edit_record,    0, NULL       },
 	  { N_("/Effec_ts"),      NULL,         NULL,           0, "<Branch>"    },
 	  { N_("/Effects/Fade _in"),NULL,       effects_fadein, 0, NULL          },
 	  { N_("/Effects/Fade o_ut"),NULL,      effects_fadeout,0, NULL          },
 	  { N_("/Effects/_Normalize"),"<control>N",effects_normalize,0, NULL     },
+	  { N_("/Effects/Normali_ze to..."),NULL,effects_normalizeto,0,NULL },
 	  { N_("/Effects/_Volume adjust (fade)..."),NULL,effects_volume,0,NULL   },
 	  { N_("/Effects/sep1"),  NULL,         NULL,           0, "<Separator>" },
 	  { N_("/Effects/Convert sample_rate..."),NULL,effects_samplerate,0,NULL },
@@ -1603,8 +1782,9 @@ static GtkWidget *create_menu(Mainwindow *w)
 	    NULL},
 	  { N_("/Effects/B_yte swap"),NULL,      effects_byteswap,0,NULL         },
 	  { N_("/Effects/sep2"),  NULL,         NULL,           0, "<Separator>" },
-	  { N_("/Effects/_Mix channels..."),NULL,effects_mixchannels,0,NULL      },
-	  { N_("/Effects/Sp_lit channel..."),NULL,effects_splitchannel,0,NULL    },
+	  { N_("/Effects/_Mix to mono"),NULL,effects_mixchannels,0,NULL      },
+	  { N_("/Effects/Add channe_l"),NULL,effects_splitchannel,0,NULL    },
+	  { N_("/Effects/Ma_p channels..."),NULL,effects_mapchannels,0,NULL },
 	  {N_("/Effects/_Combine channels..."),NULL,effects_combinechannels,0,NULL},
 	  { N_("/Effects/sep3"),  NULL,         NULL,           0, "<Separator>" },
 	  { N_("/Effects/_Speed adjustment..."),NULL,effects_speed, 0, NULL      },
@@ -1635,8 +1815,7 @@ static GtkWidget *create_menu(Mainwindow *w)
 	  "/Edit/Insert silence", 
 	  "/Edit/Select all", "/Cursor", "/View", "/Play/Play from cursor",
 	  "/Play/Play all", "/Play/Stop", "/Effects", 
-	  "/Cursor/Move to beginning", "/Cursor/Move to end", 
-	  "/Cursor/Position cursor...", "/Effects/Normalize",
+	  "/Effects/Normalize",
 	  "/Effects/Effects dialog..."
      };
 
@@ -1644,8 +1823,7 @@ static GtkWidget *create_menu(Mainwindow *w)
 	  "/Edit/Cut", "/Edit/Copy", "/Edit/Delete", "/Edit/Crop", 
 	  "/Edit/Silence selection",
 	  "/View/Zoom to selection", "/File/Save selection as...",
-	  "/Play/Play selection", "/Cursor/Move to selection start", 
-	  "/Cursor/Move to selection end"
+	  "/Play/Play selection" 
      };
 
      gchar *need_clipboard_names[] = {
@@ -1693,25 +1871,27 @@ static GtkWidget *create_menu(Mainwindow *w)
 
      for (i=0; i<ARRAY_LENGTH(need_chunk_names); i++)
 	  w->need_chunk_items = 
-	       g_list_append(w->need_chunk_items,gtk_item_factory_get_item(
+	       g_list_append(w->need_chunk_items,xgtk_item_factory_get_item(
 				  item_factory,need_chunk_names[i]));
 
      for (i=0; i<ARRAY_LENGTH(need_selection_names); i++)
 	  w->need_selection_items = 
-	       g_list_append(w->need_selection_items,gtk_item_factory_get_item(
-				  item_factory,need_selection_names[i]));
+	       g_list_append(w->need_selection_items,
+			     xgtk_item_factory_get_item(
+				 item_factory,need_selection_names[i]));
 
      for (i=0; i<ARRAY_LENGTH(need_clipboard_names); i++)
 	  w->need_clipboard_items = 
-	       g_list_append(w->need_clipboard_items,gtk_item_factory_get_item(
+	       g_list_append(w->need_clipboard_items,
+			     xgtk_item_factory_get_item(
 				  item_factory,need_clipboard_names[i]));
      
      w->need_undo_items = 
 	  g_list_append(w->need_undo_items,gtk_item_factory_get_item
 			(item_factory,"/Edit/Undo") );
      w->need_redo_items = 
-	  g_list_append(w->need_redo_items,gtk_item_factory_get_item
-			(item_factory,"/Edit/Redo"));
+	  g_list_append(w->need_redo_items,
+			xgtk_item_factory_get_item(item_factory,"/Edit/Redo"));
 
      update_file_recent(w);
 
