@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003 2004 2005, Magnus Hjorth
+ * Copyright (C) 2003 2004 2005 2006, Magnus Hjorth
  *
  * This file is part of mhWaveEdit.
  *
@@ -78,16 +78,17 @@ static gchar *samplesize_switch[] = { NULL,"-b","-w",NULL,"-l" };
 
 #define COMPAND_LINES 6
 
-void sox_dialog_format_string(gchar *buf, guint bufsize, guint samplerate,
-			      gboolean sign, guint samplesize, guint channels)
+void sox_dialog_format_string(gchar *buf, guint bufsize, Dataformat *fmt)
 {
-     g_snprintf(buf,bufsize,"-t raw -r %d %s %s -c %d",samplerate,
-		sign?"-s":"-u",samplesize_switch[samplesize],channels);
-		
+     g_assert(fmt->type == DATAFORMAT_PCM && fmt->samplesize != 3);
+
+     g_snprintf(buf,bufsize,"-t raw -r %d %s %s -c %d",fmt->samplerate,
+		fmt->sign?"-s":"-u",samplesize_switch[fmt->samplesize],
+		fmt->channels);
 }
 
-static Chunk *sox_dialog_apply_proc(Chunk *chunk, StatusBar *bar, 
-				    gpointer user_data)
+static Chunk *sox_dialog_apply_proc_main(Chunk *chunk, StatusBar *bar, 
+					 gpointer user_data)
 {
      SoxDialog *sd = SOX_DIALOG(user_data);
      EffectDialog *ed = &(sd->ed);
@@ -98,12 +99,12 @@ static Chunk *sox_dialog_apply_proc(Chunk *chunk, StatusBar *bar,
      guint i,j;
      gint idx;
      gboolean b;
+     off_t clipcount = 0;
+     Chunk *r;
 
-     sox_dialog_format_string(fmt_buf,sizeof(fmt_buf),chunk->format.samplerate,
-			      chunk->format.sign,chunk->format.samplesize,
-			      chunk->format.channels);
+     sox_dialog_format_string(fmt_buf,sizeof(fmt_buf),&(chunk->format));
      g_snprintf(cmd_buf,sizeof(cmd_buf),"sox %s - %s - ",fmt_buf,fmt_buf);
-		
+     
      c=strchr(cmd_buf,0);     
      if (!strcmp(ed->effect_name,"echo") || 
 	 !strcmp(ed->effect_name,"echos") ||
@@ -232,21 +233,47 @@ static Chunk *sox_dialog_apply_proc(Chunk *chunk, StatusBar *bar,
      } else {
 	  g_assert_not_reached();
      }
-     return pipe_dialog_pipe_chunk(chunk,cmd_buf,FALSE,dither_editing,bar);
+     r = pipe_dialog_pipe_chunk(chunk,cmd_buf,FALSE,dither_editing,bar,
+				&clipcount);
+     if (r != NULL && clipwarn(clipcount,TRUE)) {
+	  gtk_object_sink(GTK_OBJECT(r));
+	  return NULL;
+     }
+     return r;
+}
+
+static Chunk *sox_dialog_apply_proc(Chunk *chunk, StatusBar *bar, 
+				    gpointer user_data)
+{
+     Chunk *c,*d,*r;
+     Dataformat stype = { DATAFORMAT_PCM, 44100, 4, 1, 4, TRUE, IS_BIGENDIAN };
+     if ((chunk->format.type == DATAFORMAT_FLOAT) || 
+	 (chunk->format.type == DATAFORMAT_PCM && 
+	  chunk->format.samplesize == 3)) {
+	  c = chunk_convert_sampletype(chunk,&stype);
+	  d = sox_dialog_apply_proc_main(c,bar,user_data);
+	  gtk_object_sink(GTK_OBJECT(c));
+	  r = chunk_convert_sampletype(d,&(chunk->format));
+	  gtk_object_sink(GTK_OBJECT(d));
+	  return r;
+     } else
+	  return sox_dialog_apply_proc_main(chunk,bar,user_data);
 }
 
 static gboolean sox_dialog_apply(EffectDialog *ed)
 {
      SoxDialog *sd = SOX_DIALOG(ed);
      Document *d = EFFECT_BROWSER(ed->eb)->dl->selected;
-     Chunk *chunk = d->chunk;
+     /* Chunk *chunk = d->chunk; */
      guint i,j;
+     /*
      if (chunk->format.samplesize != 1 &&
 	 chunk->format.samplesize != 2 &&
 	 chunk->format.samplesize != 4) {
 	  user_info(_("SoX only supports 8, 16 and 32-bit sample sizes"));
 	  return TRUE;
      }
+     */
      if ((sd->fb1!=NULL && floatbox_check(sd->fb1)) ||
 	 (sd->fb2!=NULL && floatbox_check(sd->fb2)) || 
 	 (sd->fb3!=NULL && floatbox_check(sd->fb3)) || 
@@ -667,27 +694,37 @@ GtkType sox_dialog_get_type(void)
 
 gboolean sox_dialog_register_main(void)
 {
-     int fd[2],i,lb_pos=0;
+     int fd[2],fd2[2],i,j,lb_pos=0;
      pid_t p;
      gchar *c,*d,**s,**sn;
-     gchar linebuf[1024];
+     gchar linebuf[4096];
      gboolean *map;
      /* Run the command 'sox -h' and try to see which effects it
       * supports. */
      i = pipe(fd);
-     if (i == -1) {
+     j = pipe(fd2);
+     if (i == -1 || j == -1) {
 	  console_perror(_("Error creating pipe"));
+	  if (i == 0) { close(fd[0]); close(fd[1]); }
+	  if (j == 0) { close(fd2[0]); close(fd2[1]); }
 	  return TRUE;
      }
      p = fork();
      if (p == -1) {
 	  console_perror(_("Couldn't fork"));
+	  close(fd[0]);
+	  close(fd[1]);
+	  close(fd2[0]);
+	  close(fd2[1]);
 	  return TRUE;
      }
      if (p == 0) {
-	  /* Child process - run 'sox -h' and catch stderr output */
-	  close_all_files_except(&(fd[1]),1);
-	  if (dup2(fd[1],2)==-1 || execlp("sox","sox","-h",NULL)==-1)
+	  /* Child process - run 'sox -h' and catch stdout/stderr output */
+	  /* Put stdout descriptor in fd[0] */
+	  fd[0] = fd2[1];
+	  close_all_files_except(fd,2);
+	  if (dup2(fd[0],1)==-1 || dup2(fd[1],2)==-1 || 
+	      execlp("sox","sox","-h",NULL)==-1)
 	       
 	       printf(_("Error running 'sox -h': %s\n"),strerror(errno));
 	  else 
@@ -696,9 +733,14 @@ gboolean sox_dialog_register_main(void)
      } else {
 	  /* Parent process - read data */
 	  close(fd[1]);
-	  while (lb_pos < sizeof(linebuf)-1) {
-	       i = read(fd[0],linebuf+lb_pos,sizeof(linebuf)-lb_pos-1);
-	       if (i == 0) break;
+	  close(fd2[1]);
+	  /* Put stdout descriptor in fd array */
+	  fd[1] = fd2[0];
+	  /* Read input */
+	  j = 0;
+	  while (lb_pos < sizeof(linebuf)-1 && j<2) {
+	       i = read(fd[j],linebuf+lb_pos,sizeof(linebuf)-lb_pos-1);
+	       if (i == 0) j++; /* Read from other descriptor */
 	       if (i < 0) {
 		    if (errno == EINTR) continue;
 		    console_perror(_("Error reading sox output"));
@@ -712,10 +754,16 @@ gboolean sox_dialog_register_main(void)
 	  /* Scan for available effects */
 	  c = strstr(linebuf,"effect: ");
 	  if (c == NULL) {
-	       console_message(_("Unable to detect supported SoX effects"));
-	       return TRUE;
-	  }
-	  c += 8;
+	       c = strstr(linebuf,"effects: ");
+	       if (c == NULL) {
+		    console_message(_("Unable to detect supported "
+				      "SoX effects"));
+		    return TRUE;
+	       } else
+		    c += 9;
+	       
+	  } else
+	       c += 8;
 	  d = strchr(c,'\n');
 	  if (d) *d=0;
 	  for (d=strtok(c," "); d!=NULL; d=strtok(NULL," ")) {
